@@ -11,7 +11,7 @@ import zipfile
 from tqdm import tqdm
 from tqdm.utils import CallbackIOWrapper
 
-from .utils import zipdir, getApiKey, run_job, new_job, get_api_host, generate_requirements_file, auth, set_api_host,  get_model, new_version, deploy_version, find_version
+from .utils import zipdir, add_files_to_zip, getApiKey, run_job, new_job, get_api_host, generate_requirements_file, auth, set_api_host,  get_model, new_version, deploy_version, find_version
 
 settings = EasySettings(str(Path.home()) + "/.inferrd.conf")
 
@@ -61,6 +61,7 @@ def get_requests(name, **kwargs):
 
   return r.json()
 
+# ------ JOBS
 def run(team_name, job, **kwargs):
   if(getApiKey() == ''):
     raise Exception('No api key. Use inferrd.auth() first.')
@@ -95,6 +96,9 @@ def run(team_name, job, **kwargs):
   zipf.write('./job.dill', './job.dill')
   zipf.write('./reqs.txt', './requirements.txt')
 
+  if 'includeFiles' in kwargs:
+    add_files_to_zip(zipf, include=kwargs['includeFiles'])
+
   if 'setup' in kwargs:
     setupFile = open('./setup.sh', 'w')
     setupFile.write(kwargs['setup'])
@@ -128,6 +132,7 @@ def run(team_name, job, **kwargs):
   if 'setup' in kwargs:
     os.remove('./setup.sh')
 
+# ------ CUSTOM MODEL
 def deploy(model, **kwargs):
   if(getApiKey() == ''):
     raise Exception('No api key. Use inferrd.auth() first.')
@@ -160,6 +165,9 @@ def deploy(model, **kwargs):
   zipf.write('./model.dill', './model.dill')
   zipf.write('./reqs.txt', './requirements.txt')
 
+  if 'includeFiles' in kwargs:
+    add_files_to_zip(zipf, include=kwargs['includeFiles'])
+
   if 'setup' in kwargs:
     setupFile = open('./setup.sh', 'w')
     setupFile.write(kwargs['setup'])
@@ -187,8 +195,7 @@ def deploy(model, **kwargs):
   if 'setup' in kwargs:
     os.remove('./setup.sh')
 
-  print('Your model is now deploying! Try it out using this code:\n')
-  print('inferrd.get(\'{0}\')([1,2])'.format(name))
+  print('> Your model is now deploying!')
 
 
 # ------ TENSORFLOW
@@ -288,15 +295,13 @@ def deploy_spacy(nlp_model, name):
   if(getApiKey() == ''):
     raise Exception('No api key. Use inferrd.auth() first.')
 
-  print('> Saving model to folder')
+  print('> Saving model to disk')
 
   if nlp_model is None:
     raise Exception('Empty spaCy model. Make sure the first argument is a spaCy model.')
 
   model = get_model(name)
 
-  version = new_version(model['id'])
-
   if os.path.exists('./inferrd-model'):
     shutil.rmtree('./inferrd-model')
 
@@ -312,6 +317,7 @@ def deploy_spacy(nlp_model, name):
   zipf.close()
 
   # upload to storage
+  version = new_version(model['id'])
   model_size = Path('./model.zip').stat().st_size
   print('> Uploading model ({0})'.format(sizeof_fmt(model_size)), flush=True)
   f = open("./model.zip", 'rb')
@@ -324,10 +330,56 @@ def deploy_spacy(nlp_model, name):
   deploy_version(version['id'])
 
   shutil.rmtree('./inferrd-model')
-  #os.remove('./model.zip')
+  os.remove('./model.zip')
 
   print('> spaCy model deployed')
   
+def deploy_keras(keras_model, name):
+  if(getApiKey() == ''):
+    raise Exception('No api key. Use inferrd.auth() first.')
+
+  if keras_model is None:
+    raise Exception('Empty Kears model. Make sure the first argument is a Keras model.')
+
+  print('> Saving model to disk')
+
+  from tensorflow import keras
+
+  model = get_model(name)
+
+  if os.path.exists('./_inferrd_keras.h5'):
+    os.remove('./_inferrd_keras.h5')
+
+  # by default use the h5 format, otherwise it'd be a tensorflow deploy
+  keras_model.save('_inferrd_keras.h5', save_format='h5')
+
+  print('> Zipping model for upload')
+
+  if os.path.exists('./model.zip'):
+    os.remove('./model.zip')
+
+  zipf = zipfile.ZipFile('model.zip', 'w', zipfile.ZIP_DEFLATED)
+  zipf.write('./_inferrd_keras.h5', './model.h5')
+  zipf.close()
+
+  # upload to storage
+  version = new_version(model['id'])
+  model_size = Path('./model.zip').stat().st_size
+  print('> Uploading model ({0})'.format(sizeof_fmt(model_size)), flush=True)
+  f = open("./model.zip", 'rb')
+
+  with tqdm(total=model_size, unit="B", unit_scale=True, unit_divisor=1024, ncols=100) as t:
+    wrapped_file = CallbackIOWrapper(t.update, f, "read")
+    r = requests.put(version['signedUpload'], data=wrapped_file, headers={'Content-Type': 'application/zip'})
+
+  print('> Deploying version v' + str(version['number']))
+  deploy_version(version['id'])
+  os.remove('./model.zip')
+  os.remove('./_inferrd_keras.h5')
+
+  print('> spaCy model deployed')
+
+
 def get_request_history(apiKey, kwargs):
   print('Getting request')
 
